'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.extractHandlers = extractHandlers;
exports.get = get;
exports.register = register;
exports.create = create;
exports.update = update;
exports.attachListeners = attachListeners;
exports.updateChildren = updateChildren;

var _titaniumPlatformsOsIos = require('titanium-platforms/os/ios');

var _titaniumPlatformsOsIos2 = _interopRequireDefault(_titaniumPlatformsOsIos);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var assign = _Object$assign;

// Utilities

var handlerRE = /^on[A-Z]/g;

function extractHandlers(props) {
  var handlers = {};
  var rest = {};

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = _getIterator(_Object$keys(props)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var key = _step.value;

      var value = props[key];

      if (typeof value === 'function' && key.match(handlerRE)) {
        handlers[key.slice(2, 3).toLowerCase() + key.slice(3)] = value;
      } else {
        rest[key] = value;
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator['return']) {
        _iterator['return']();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return { handlers: handlers, rest: rest };
}

// Definitions

var registry = {};

function get(type) {
  (0, _invariant2['default'])(type in registry, 'No definition found for "' + type + '"');

  return registry[type];
}

var defaults = {
  factory: function factory(props) {
    return Ti.UI.createView(props);
  },

  create: function create(props, handlers, getChildren) {
    var view = this.factory(props);

    attachListeners(view, handlers);

    updateChildren(view, getChildren());

    return view;
  },

  update: function update(view, props, handlers) {
    // TODO: manage handlers

    for (var key in props) {
      var nextValue = props[key];

      if (key === 'value') {
        var oldValue = view[key];

        if (nextValue === oldValue) {
          continue;
        }
      }

      view[key] = nextValue;
    }

    // view.applyProperties(props);
  }
};

function register(shortName, apiName) {
  var config = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var definition = _extends({}, defaults, {
    shortName: shortName,
    apiName: apiName
  }, config);

  registry[shortName] = definition;
  registry[apiName] = definition;

  return definition;
}

function create(type, props, handlers, getChildren) {
  return get(type).create(props, handlers, getChildren);
}

function update(type, view, props, handlers) {
  return get(type).update(view, props, handlers);
}

function attachListeners(view, handlers) {
  for (var _name in handlers) {
    view.addEventListener(_name, handlers[_name]);
  }
}

function updateChildren(view, children) {
  if (_titaniumPlatformsOsIos2['default']) {
    view.add(children);
  } else {
    view.removeAllChildren();

    // NOTE: Sloooooooow...

    var i = 0;
    var l = children.length;

    for (; i < l; ++i) {
      if (children[i]) {
        view.add(children[i]);
      }
    }
  }
}